{
  Attach files to a VGA Planets 3.2x TRN file

  Usage: attach playerx.trn       List attachments of TRN files
         attach playerx.trn file  Attach specified file to the TRN
                                  (up to 10 per TRN file)

  This is a 30 minute hack, so neither bullet-proof nor elegant.
  It was tested with some hosts between 3.20beta and 3.22.something.

  Compiler: Turbo Pascal 6, other versions should work, too.

  Written in Apr/1998 by Stefan Reuther <Streu@gmx.de>; English
  comments added Feb/2001

  Public Domain, no warranty whatsoever
}

PROGRAM Attach;

{--- Header of modified TRN, without signature ---}
TYPE AttachHdr=RECORD
       TRNPos,TRNSize:LONGINT;          
       Attached:ARRAY[1..10] OF RECORD
          Pos,Size:LONGINT;
          Name:ARRAY[1..12] OF CHAR;
       END;
     END;

{--- "forbidden" extensions (rejected by host) ---}
CONST MAXNO=9;
      NONO:ARRAY[1..MAXNO] OF STRING[3]=('HST','INI','EXT','DAT','TRN','NM',
                                         'RST','LOG','BAT');

{--- Signature ---}
CONST Signature='NCC1701AD9';

VAR F:FILE;                             { TRN file }
    Sign:ARRAY[1..10] OF CHAR;          { Signature buffer }
    Buf:ARRAY[0..1023] OF BYTE;         { Copy buffer }
    H:AttachHdr;
    i,j,k:INTEGER;
    S:STRING;
    S1:STRING[3];

    Size:LONGINT;

{
  TRUE iff c is a path separator
}
FUNCTION IsPathSep(c:CHAR):BOOLEAN;
BEGIN
  IsPathSep:=(c='\') OR (c=':');
END;

{
  Replace file name in N by Newname, keep existing path
}
PROCEDURE ReplaceFileName(VAR N:STRING; NewName:STRING);
BEGIN
  WHILE (N<>'') AND NOT IsPathSep(N[Length(N)]) DO Dec(N[0]);
  N:=N+NewName;
END;

{
  Copy file S into the turn file, erase it afterwards if Era=TRUE
}
PROCEDURE CopyFrom(S:STRING; Era:BOOLEAN);
VAR R,W:WORD;
    F1:FILE;
BEGIN
 {$I-}
  Assign(F1,S);
  Reset(F1,1);
 {$I+}
  IF IOResult<>0 THEN BEGIN
    Writeln('ATTACH: Can''t open '+S);
    Halt;
  END;
  Size:=FileSize(F1);
  REPEAT
    Blockread(F1,Buf,1024,R);
    IF R<>0 THEN Blockwrite(F,Buf,R,W);
  UNTIL R=0;
  Close(F1);
  IF Era THEN Erase(F1);
END;

{
  Modify TRN so we can attach files
}
PROCEDURE MakeAttachable;
VAR S:STRING;
BEGIN
  FillChar(H,Sizeof(H),0);
  H.TrnSize := FileSize(F);
  H.TrnPos := 11+Sizeof(H);
  Close(F);

  S:=ParamStr(1);
  ReplaceFilename(S,'attach.$$$');
  Rename(F,S);
  Sign:=Signature;
  Assign(F,ParamStr(1));
  Rewrite(F,1);
  BlockWrite(F,Sign,10);
  BlockWrite(F,H,Sizeof(H));
  CopyFrom(S,TRUE);
END;

BEGIN
  Writeln('VGA Planets(tm) 3.2x Turn File Attachments');
  IF ParamCount=0 THEN BEGIN
    Writeln('Usage: ATTACH playerx.trn            - to get a list of all attached files.');
    Writeln('       ATTACH playerx.trn file       - to attach a file to the TRN file');
    Halt;
  END;
  Assign(F,ParamStr(1));
  {$I-}
  Reset(F,1);
  {$I+}
  IF IOResult<>0 THEN BEGIN
    Writeln('ATTACH: Can''t open ',ParamStr(1));
    Halt;
  END;
  IF ParamCount=1 THEN BEGIN
    Writeln('Contents of ',ParamStr(1),':');
  END;
  Blockread(F,Sign,10);

  IF Sign<>Signature THEN BEGIN         { Is this TRN `attach-ready'? }
    IF ParamCount=1 THEN BEGIN          { No -> say so if we're just reading }
      Close(F);
      Writeln('0 attachments.');
      Halt;
    END;
    MakeAttachable;                     { convert it if we want to attach sth. }
  END ELSE BEGIN
    Seek(F,10);                         { Yes, it is -> read header }
    Blockread(F,H,Sizeof(H));
    IF ParamCount=1 THEN BEGIN          { listing only }
      j:=0;
      FOR i:=1 TO 10 DO IF H.Attached[i].Size OR H.Attached[i].Pos<>0 THEN BEGIN
        Writeln(H.Attached[i].Name,H.Attached[i].Size:10);
        Inc(j);
      END;
      Writeln(j,' attachments.');
      Close(F);
      Halt;
    END;
  END;

  j:=0;                                 { look for empty slot }
  FOR i:=10 DOWNTO 1 DO IF H.Attached[i].Size OR H.Attached[i].Pos=0 THEN j:=i;
  IF j=0 THEN BEGIN
    Writeln('ATTACH: Maximum attachment count (10) reached!');
    Close(F);
    Halt;
  END;

  S:=ParamStr(2);                       { Is this a "forbidden file"? }
  WHILE (Pos('\',S) OR Pos(':',S)<>0) DO Delete(S,1,1);
  i:=Pos('.',S);
  IF i>0 THEN BEGIN
    S1:=Copy(S,i+1,3);
    WHILE (S1<>'') AND (S1[Length(S1)]=' ') DO Dec(S1[0]);
    FOR i:=1 TO Length(S1) DO S1[i]:=Upcase(S1[i]);
    FOR i:=1 TO MAXNO DO IF S1=NONO[i] THEN BEGIN
      Writeln('ATTACH: You can''t send ',S1,' files.');
      Close(F);
      Halt;
    END;
  END;

  H.Attached[j].Pos := FileSize(F)+1;   { Embed file }
  Seek(F,H.Attached[j].Pos-1);
  CopyFrom(ParamStr(2),FALSE);
  H.Attached[j].Size := Size;
  WHILE Length(S)<12 DO S:=S+' ';
  Move(S[1], H.Attached[j].Name, 12);
  Seek(F,10);
  Blockwrite(F,H,Sizeof(H));
  Close(F);
  Writeln('File ',S,' (',Size,' bytes) attached to TRN file, slot ',j);
END.
